/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package org.ow2.petals.tools.jmx.api;

import org.ow2.petals.tools.jmx.api.exception.AdminDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.AdminServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.ComponentComponentDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.ComponentErrorException;
import org.ow2.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.ow2.petals.tools.jmx.api.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.InstallationServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.PetalsAdminServiceErrorException;

/**
 * The Petals JMX Client API
 * 
 * @author cdeneux - Capgemini Sud
 */
public final class PetalsJMXClient {

    private final static String PETALS_DOMAIN = "Petals";

    private PetalsJMXConnection jmxConnection = null;

    /**
     * Creates a new instance of {@link PetalsJMXClient}
     * 
     * @param host
     * @param port
     * @param username
     * @param password
     * @throws ConnectionErrorException
     */
    public PetalsJMXClient(String host, Integer port, String username, String password)
            throws ConnectionErrorException {
        this.jmxConnection = new PetalsJMXConnection(host, port, username, password);
    }

    /**
     * Retrieve the Petals Admin service client API
     * 
     * @return the Petals Admin service client API
     * @throws PetalsAdminDoesNotExistException
     * @throws PetalsAdminServiceErrorException
     * @throws ConnectionErrorException
     */
    public PetalsAdminServiceClient getPetalsAdminServiceClient()
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException,
            ConnectionErrorException {

        return new PetalsAdminServiceClient(PETALS_DOMAIN, jmxConnection.getMBeanServerConnection());

    }

    /**
     * Retrieve the Admin service client API
     * 
     * @return the Admin service client API
     * @throws AdminDoesNotExistException
     * @throws AdminServiceErrorException
     * @throws ConnectionErrorException
     */
    public AdminServiceClient getAdminServiceClient() throws AdminDoesNotExistException,
            AdminServiceErrorException, ConnectionErrorException {

        return new AdminServiceClient(PETALS_DOMAIN, jmxConnection.getMBeanServerConnection());

    }

    /**
     * Retrieve the installation service client API
     * 
     * @return the installation service client API
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     * @throws ConnectionErrorException
     */
    public PetalsInstallationServiceClient getInstallationServiceClient()
            throws InstallationServiceDoesNotExistException, InstallationServiceErrorException,
            ConnectionErrorException {

        return new PetalsInstallationServiceClient(PETALS_DOMAIN, jmxConnection
                .getMBeanServerConnection());

    }

    /**
     * Retrieve the deployment service client API
     * 
     * @return the deployment service client API
     * @throws DeploymentServiceDoesNotExistException
     * @throws DeploymentServiceErrorException
     * @throws ConnectionErrorException
     */
    public PetalsDeploymentServiceClient getDeploymentServiceClient()
            throws DeploymentServiceDoesNotExistException, DeploymentServiceErrorException,
            ConnectionErrorException {

        return new PetalsDeploymentServiceClient(PETALS_DOMAIN, jmxConnection
                .getMBeanServerConnection());

    }

    /**
     * Retrieve the component client API associated to a JBI component.
     * 
     * @param componentName
     *            Component name
     * @return the component client API
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     * @throws ConnectionErrorException
     */
    public PetalsComponentComponentClient getComponentComponentClient(String componentName)
            throws ComponentComponentDoesNotExistException, ComponentErrorException,
            ConnectionErrorException {

        return new PetalsComponentComponentClient(PETALS_DOMAIN, componentName, jmxConnection
                .getMBeanServerConnection());

    }

    /**
     * Retrieve the component installer client API associated to a JBI
     * component.
     * 
     * @param componentName
     *            Component name
     * @return the installer component client API
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     * @throws ConnectionErrorException
     */
    public PetalsInstallerComponentClient getInstallerComponentClient(String componentName)
            throws InstallerComponentDoesNotExistException, ComponentErrorException,
            ConnectionErrorException {

        return new PetalsInstallerComponentClient(PETALS_DOMAIN, componentName, jmxConnection
                .getMBeanServerConnection());

    }

}
